#include <stdio.h>
#include <avr\io.h>
#define F_CPU 32000000UL
#include <util\delay.h>
#include <inttypes.h>

/********************** initialization **************************************************/
void Config32MHzClock(void)
{
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  // initialize clock source to be 32MHz internal oscillator (no PLL)
  OSC.CTRL = OSC_RC32MEN_bm; // enable internal 32MHz oscillator
  while(!(OSC.STATUS & OSC_RC32MRDY_bm)); // wait for oscillator ready
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  CLK.CTRL = 0x01; //select sysclock 32MHz osc
}

void init(void)
{
  Config32MHzClock();

  CLK.PSCTRL = 0x00; 			// no division on peripheral clock

  PORTC.DIR |= (1<<3) | (1<<0); // set PORTC:3 transmit pin as output
  PORTC.OUT |= (1<<3);          // set PORTC:3 hi 
  USARTC0.BAUDCTRLA = 207; 		// 9600b  (BSCALE=207,BSEL=0)

  USARTC0.CTRLB = USART_TXEN_bm | USART_RXEN_bm; // enable tx and rx on USART

  PORTE.DIR = 0xFF; // led port
  PORTE.OUT = 0xFF; // leds off
}

/********************** serial communication ********************************************/
void chPutChar(uint8_t zeichen)
{
    USARTC0.DATA = zeichen; 						// transmit 
	if(!(USARTC0.STATUS&USART_DREIF_bm))
		while(!(USARTC0.STATUS & USART_TXCIF_bm));	// wait for TX complete
  	USARTC0.STATUS |= USART_TXCIF_bm;  				// clear TX interrupt flag
}
uint8_t chGetChar()
{

	while(!(USARTC0.STATUS&USART_RXCIF_bm));  		// wait for RX complete
	PORTE.OUTTGL = (1<<7); 							// toogle indicator led

  	return USARTC0.DATA;
}
uint8_t chCharReady()
{
	return (USARTC0.STATUS&USART_RXCIF_bm);
}

void printString(char *data)
{
	unsigned char i = 0;
	while(data[i]!=0x00) chPutChar(data[i++]);
}

void PrintInt(int16_t wert)
{  	
	char text[6];
	itoa(wert,text,10);
	printString(text);
}
void PrintTextInt(char *str,int16_t wert)
{  	
	char text[6];
	printString(str);
	PrintInt(wert);
	printString("\n\r");
}

initADC()
{
	//************ ADC on port A ****************************************************************

	ADCA.CTRLA |= 0x1;                                   	// enable adc
	ADCA.CTRLB = ADC_RESOLUTION_12BIT_gc;                	// 12 bit conversion
	//ADCA.REFCTRL = ADC_REFSEL_INT1V_gc | 0x02;        	// internal 1V bandgap reference
	ADCA.REFCTRL = ADC_REFSEL_VCC_gc | 0x02;   				// internal VCC-0.6V
	ADCA.PRESCALER = ADC_PRESCALER_DIV128_gc;             	// adc clk (32MHz/x=ykHz)
	ADCA.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;    	// single ended
	ADCA.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc;           	// select some input pin

	//************ ADC on port B ****************************************************************

	ADCB.CTRLA |= 0x1;                                   	// enable adc
	ADCB.CTRLB = ADC_RESOLUTION_12BIT_gc;                	// 12 bit conversion
	//ADCB.REFCTRL = ADC_REFSEL_INT1V_gc | 0x02;        	// internal 1V bandgap reference
	ADCB.REFCTRL = ADC_REFSEL_VCC_gc | 0x02;   				// internal VCC-0.6V
	ADCB.PRESCALER = ADC_PRESCALER_DIV128_gc;             	// adc clk (32MHz/x=ykHz)
	ADCB.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;    	// single ended
	ADCB.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc;           	// select some input pin
}
/*
	pin assignement on XPLAIN board
	===============================

	readADC channel		port		description
	0-7					PA0-PA7: 	analog inputs
	16					PB0: 		NTC input
	17					PB1: 		poti input

*/
int16_t readADC(uint8_t channel)
{
	int16_t analogWert;
	uint8_t chan=0;
	register8_t select;

	chan=channel&0x0F;

	switch(chan)
	{
		case 0:
		{
			select=ADC_CH_MUXPOS_PIN0_gc;
		}break;
		case 1:
		{
			select=ADC_CH_MUXPOS_PIN1_gc;
		}break;
		case 2:
		{
			select=ADC_CH_MUXPOS_PIN2_gc;
		}break;
		case 3:
		{
			select=ADC_CH_MUXPOS_PIN3_gc;
		}break;
		case 4:
		{
			select=ADC_CH_MUXPOS_PIN4_gc;
		}break;
		case 5:
		{
			select=ADC_CH_MUXPOS_PIN5_gc;
		}break;
		case 6:
		{
			select=ADC_CH_MUXPOS_PIN6_gc;
		}break;
		case 7:
		{
			select=ADC_CH_MUXPOS_PIN7_gc;
		}break;

		default:select=ADC_CH_MUXPOS_PIN0_gc;
	}
	if(channel==16)
	{
	    PORTQ.DIR = 0B00000100; //enable ntc ON XPLAIN
		PORTQ.OUT = 0b00000000; //ENABLE ntc ON XPLAIN
	}
	else PORTQ.OUT = 0b00000100; //DISABLE ntc ON XPLAIN;

	//************ ADC on port A ************
	if(channel<16)
	{

		ADCA.CH0.MUXCTRL = select;	// PORTA

		// trigger single conversion
		ADCA.CTRLA |= 0x4; 
		// wait for result
		while(!ADCA.CH0.INTFLAGS);			// wait for conversion complete flag

		analogWert = ADCA.CH0RES; 			// read 12 bit value 
	}
	//************ ADC on port B ************
	else 
	{

		ADCB.CH0.MUXCTRL = select;	// PORTB

		// trigger single conversion
		ADCB.CTRLA |= 0x4; 
		// wait for result
		while(!ADCB.CH0.INTFLAGS);			// wait for conversion complete flag

		analogWert = ADCB.CH0RES; 			// read 12 bit value 
	}

	return analogWert;
}

/********************** main programm ***************************************************/

int main(void)
{
	int data;
	uint8_t n;
	uint32_t adr;
	uint8_t wert;

	init();
	initADC();

	printString("XPLAIN ADC TEST \n\r");

	while(1)
	{
		PrintTextInt("ADC_Wert: ",readADC(16)); //16=NTC

		_delay_ms(200);
	}
}

